@@ -1,7 +1,6 @@
 """Implements the CmdContext and Command classes.
 
-- A CmdContext creates new command objects. You should use the `context`
-function to create these, rather than creating them directly.
+- A CmdContext creates new command objects.
 
 - A Command specifies the arguments and options used to run a program.
 """
@@ -10,27 +9,40 @@ import asyncio
 import dataclasses
 import enum
 import os
+import shutil
 import signal
 from collections.abc import Iterable
 from dataclasses import dataclass, field
+from pathlib import Path
 from types import TracebackType
 from typing import (
     Any,
+    AsyncIterator,
     Callable,
     ClassVar,
     Container,
+    Coroutine,
+    Generator,
+    Generic,
     Optional,
     Sequence,
+    TypedDict,
     TypeVar,
     Union,
+    cast,
+    overload,
 )
-from warnings import warn
-
-from typing_extensions import Self
 
 import shellous
 from shellous.pty_util import PtyAdapterOrBool
-from shellous.redirect import STDIN_TYPES, STDOUT_APPEND_TYPES, STDOUT_TYPES, Redirect
+from shellous.redirect import (
+    STDIN_TYPES,
+    STDOUT_TYPES,
+    Redirect,
+    StdinType,
+    StdoutType,
+    aiter_preflight,
+)
 from shellous.runner import Runner
 from shellous.util import EnvironmentDict, context_aenter, context_aexit
 
@@ -42,38 +54,52 @@ class _UnsetEnum(enum.Enum):
     UNSET = enum.auto()
 
 
-# _UnsetEnum = enum.Enum("UNSET", "UNSET")
 _UNSET = _UnsetEnum.UNSET
 
 # Use Unset[T] for unset variable types.
 _T = TypeVar("_T")
 Unset = Union[_T, _UnsetEnum]
 
-_Redirect_T = Any  # type: ignore
-_Preexec_Fn_T = Optional[Callable[[], None]]  # pylint: disable=invalid-name
-_AuditDict = dict[str, Any]  # TODO: use TypedDict?
-_Audit_Fn_T = Optional[
-    Callable[[str, _AuditDict], None]
-]  # pylint: disable=invalid-name
+_RedirectT = Any  # type: ignore
+_PreexecFnT = Optional[Callable[[], None]]
+
+
+class AuditEventInfo(TypedDict):
+    "Info attached to each Audit Event."
+
+    runner: Runner
+    "Reference to the Runner object."
+
+    failure: str
+    "When phase is 'stop', the name of the exception from starting the process."
+
+    signal: str
+    "When phase is 'signal', the signal name/number sent to the process."
+
+
+_AuditFnT = Optional[Callable[[str, AuditEventInfo], None]]
 
 
 @dataclass(frozen=True)
 class Options:  # pylint: disable=too-many-instance-attributes
     "Concrete class for per-command options."
 
+    path: Optional[str] = None
+    "Optional search path to use instead of PATH environment variable."
+
     env: Optional[EnvironmentDict] = field(default=None, repr=False)
     "Additional environment variables for command."
 
     inherit_env: bool = True
     "True if subprocess should inherit the current environment variables."
 
-    input: _Redirect_T = Redirect.DEFAULT
+    input: _RedirectT = Redirect.DEFAULT
     "Input object to bind to stdin."
 
     input_close: bool = False
     "True if input object should be closed after subprocess launch."
 
-    output: _Redirect_T = Redirect.DEFAULT
+    output: _RedirectT = Redirect.DEFAULT
     "Output object to bind to stdout."
 
     output_append: bool = False
@@ -82,7 +108,7 @@ class Options:  # pylint: disable=too-many-instance-attributes
     output_close: bool = False
     "True if output object should be closed after subprocess launch."
 
-    error: _Redirect_T = Redirect.DEFAULT
+    error: _RedirectT = Redirect.DEFAULT
     "Error object to bind to stderr."
 
     error_append: bool = False
@@ -91,13 +117,13 @@ class Options:  # pylint: disable=too-many-instance-attributes
     error_close: bool = False
     "True if error object should be closed after subprocess launch."
 
-    encoding: Optional[str] = "utf-8"
-    "Specifies encoding of input/ouput. None means binary."
+    encoding: str = "utf-8"
+    "Specifies encoding of input/ouput."
 
-    return_result: bool = False
+    _return_result: bool = False
     "True if we should return `Result` object instead of the output text/bytes."
 
-    catch_cancelled_error: bool = False
+    _catch_cancelled_error: bool = False
     "True if we should raise `ResultError` after clean up from cancelled task."
 
     exit_codes: Optional[Container[int]] = None
@@ -121,13 +147,13 @@ class Options:  # pylint: disable=too-many-instance-attributes
     pass_fds_close: bool = False
     "True if pass_fds should be closed after subprocess launch."
 
-    writable: bool = False
+    _writable: bool = False
     "True if using process substitution in write mode."
 
     _start_new_session: bool = False
     "True if child process should start a new session with setsid()."
 
-    _preexec_fn: _Preexec_Fn_T = None
+    _preexec_fn: _PreexecFnT = None
     "Function to call in child process after fork from parent."
 
     pty: PtyAdapterOrBool = False
@@ -136,7 +162,7 @@ class Options:  # pylint: disable=too-many-instance-attributes
     close_fds: bool = False
     "True if child process should close all file descriptors."
 
-    audit_callback: _Audit_Fn_T = None
+    audit_callback: _AuditFnT = None
     "Function called to audit stages of process execution."
 
     def merge_env(self) -> Optional[dict[str, str]]:
@@ -150,9 +176,11 @@ class Options:  # pylint: disable=too-many-instance-attributes
             return dict(self.env)  # make copy of dict
         return {}
 
-    def set_stdin(self, input_: Any, close: bool) -> Self:
+    def set_stdin(self, input_: Any, close: bool) -> "Options":
         "Return new options with `input` configured."
-        input_ = Redirect.from_literal(input_)
+        if input_ is None:
+            raise TypeError("invalid stdin")
+
         if input_ == Redirect.STDOUT:
             raise ValueError("STDOUT is only supported by stderr")
 
@@ -162,9 +190,11 @@ class Options:  # pylint: disable=too-many-instance-attributes
             input_close=close,
         )
 
-    def set_stdout(self, output: Any, append: bool, close: bool) -> Self:
+    def set_stdout(self, output: Any, append: bool, close: bool) -> "Options":
         "Return new options with `output` configured."
-        output = Redirect.from_literal(output)
+        if output is None:
+            raise TypeError("invalid stdout")
+
         if output == Redirect.STDOUT:
             raise ValueError("STDOUT is only supported by stderr")
 
@@ -175,9 +205,11 @@ class Options:  # pylint: disable=too-many-instance-attributes
             output_close=close,
         )
 
-    def set_stderr(self, error: Any, append: bool, close: bool) -> Self:
+    def set_stderr(self, error: Any, append: bool, close: bool) -> "Options":
         "Return new options with `error` configured."
-        error = Redirect.from_literal(error, True)
+        if error is None:
+            raise TypeError("invalid stderr")
+
         return dataclasses.replace(
             self,
             error=error,
@@ -185,12 +217,12 @@ class Options:  # pylint: disable=too-many-instance-attributes
             error_close=close,
         )
 
-    def set_env(self, updates: dict[str, Any]) -> Self:
+    def set_env(self, updates: dict[str, Any]) -> "Options":
         "Return new options with augmented environment."
         new_env = EnvironmentDict(self.env, updates)
         return dataclasses.replace(self, env=new_env)
 
-    def set(self, kwds: dict[str, Any]) -> Self:
+    def set(self, kwds: dict[str, Any]) -> "Options":
         """Return new options with given properties updated.
 
         See `Command.set` for option reference.
@@ -198,9 +230,25 @@ class Options:  # pylint: disable=too-many-instance-attributes
         kwds = {key: value for key, value in kwds.items() if value is not _UNSET}
         return dataclasses.replace(self, **kwds)
 
+    @overload
+    def which(self, name: bytes) -> Optional[bytes]:
+        "Find the command with the given name and return its path."
+
+    @overload
+    def which(self, name: str) -> Optional[str]:
+        "Find the command with the given name and return its path."
+
+    def which(self, name: Union[str, bytes]) -> Optional[Union[str, bytes]]:
+        "Find the command with the given name and return its path."
+        return shutil.which(name, path=self.path)
+
+
+# Return type for a Command, CmdContext can be either `str` or `Result`.
+_RT = TypeVar("_RT", str, "shellous.Result")
+
 
 @dataclass(frozen=True)
-class CmdContext:
+class CmdContext(Generic[_RT]):
     """Concrete class for an immutable execution context."""
 
     CAPTURE: ClassVar[Redirect] = Redirect.CAPTURE
@@ -215,28 +263,45 @@ class CmdContext:
     STDOUT: ClassVar[Redirect] = Redirect.STDOUT
     "Redirect stderr to same place as stdout."
 
-    RESULT: ClassVar[Redirect] = Redirect.RESULT
-    "Redirect stderr to a buffer in the Result object."
+    BUFFER: ClassVar[Redirect] = Redirect.BUFFER
+    "Redirect output to a buffer in the Result object. This is the default for stdout/stderr."
 
-    options: Options = Options()
+    options: Options = field(default_factory=Options)
     "Default command options."
 
-    def stdin(self, input_: Any, *, close: bool = False) -> Self:
+    def stdin(
+        self,
+        input_: Any,
+        *,
+        close: bool = False,
+    ) -> "CmdContext[_RT]":
         "Return new context with updated `input` settings."
         new_options = self.options.set_stdin(input_, close)
         return CmdContext(new_options)
 
-    def stdout(self, output: Any, *, append: bool = False, close: bool = False) -> Self:
+    def stdout(
+        self,
+        output: Any,
+        *,
+        append: bool = False,
+        close: bool = False,
+    ) -> "CmdContext[_RT]":
         "Return new context with updated `output` settings."
         new_options = self.options.set_stdout(output, append, close)
         return CmdContext(new_options)
 
-    def stderr(self, error: Any, *, append: bool = False, close: bool = False) -> Self:
+    def stderr(
+        self,
+        error: Any,
+        *,
+        append: bool = False,
+        close: bool = False,
+    ) -> "CmdContext[_RT]":
         "Return new context with updated `error` settings."
         new_options = self.options.set_stderr(error, append, close)
         return CmdContext(new_options)
 
-    def env(self, **kwds: Any) -> Self:
+    def env(self, **kwds: Any) -> "CmdContext[_RT]":
         """Return new context with augmented environment."""
         new_options = self.options.set_env(kwds)
         return CmdContext(new_options)
@@ -244,10 +309,11 @@ class CmdContext:
     def set(  # pylint: disable=unused-argument, too-many-locals
         self,
         *,
+        path: Unset[Optional[str]] = _UNSET,
         inherit_env: Unset[bool] = _UNSET,
-        encoding: Unset[Optional[str]] = _UNSET,
-        return_result: Unset[bool] = _UNSET,
-        catch_cancelled_error: Unset[bool] = _UNSET,
+        encoding: Unset[str] = _UNSET,
+        _return_result: Unset[bool] = _UNSET,
+        _catch_cancelled_error: Unset[bool] = _UNSET,
         exit_codes: Unset[Optional[Container[int]]] = _UNSET,
         timeout: Unset[Optional[float]] = _UNSET,
         cancel_timeout: Unset[float] = _UNSET,
@@ -255,33 +321,55 @@ class CmdContext:
         alt_name: Unset[Optional[str]] = _UNSET,
         pass_fds: Unset[Iterable[int]] = _UNSET,
         pass_fds_close: Unset[bool] = _UNSET,
-        writable: Unset[bool] = _UNSET,
+        _writable: Unset[bool] = _UNSET,
         _start_new_session: Unset[bool] = _UNSET,
-        _preexec_fn: Unset[_Preexec_Fn_T] = _UNSET,
+        _preexec_fn: Unset[_PreexecFnT] = _UNSET,
         pty: Unset[PtyAdapterOrBool] = _UNSET,
         close_fds: Unset[bool] = _UNSET,
-        audit_callback: Unset[_Audit_Fn_T] = _UNSET,
-    ) -> Self:
+        audit_callback: Unset[_AuditFnT] = _UNSET,
+    ) -> "CmdContext[_RT]":
         """Return new context with custom options set.
 
         See `Command.set` for option reference.
         """
         kwargs = locals()
         del kwargs["self"]
-        if encoding is None:
-            warn(
-                "encoding=None is deprecated; use encoding='latin1' instead",
-                DeprecationWarning,
-            )
+        if not encoding:
+            raise TypeError("invalid encoding")
         return CmdContext(self.options.set(kwargs))
 
-    def __call__(self, *args: Any) -> "Command":
+    def __call__(self, *args: Any) -> "Command[_RT]":
         "Construct a new command."
         return Command(coerce(args), self.options)
 
+    @property
+    def result(self) -> "CmdContext[shellous.Result]":
+        "Set `_return_result` and `exit_codes`."
+        return cast(
+            CmdContext[shellous.Result],
+            self.set(
+                _return_result=True,
+                exit_codes=range(-255, 256),
+            ),
+        )
+
+    def find_command(self, name: str) -> Optional[Path]:
+        """Find the command with the given name and return its filesystem path.
+
+        Return None if the command name is not found in the search path.
+
+        Use the `path` variable specified by the context if set. Otherwise, the
+        default behavior is to use the `PATH` environment variable with a
+        fallback to the value of `os.defpath`.
+        """
+        result = self.options.which(name)
+        if not result:
+            return None
+        return Path(result)
+
 
 @dataclass(frozen=True)
-class Command:
+class Command(Generic[_RT]):
     """A Command instance is lightweight and immutable object that specifies the
     arguments and options used to run a program. Commands do not do anything
     until they are awaited.
@@ -299,13 +387,13 @@ class Command:
     ```
     """
 
-    args: "tuple[Union[str, bytes, Command, shellous.Pipeline], ...]"
+    args: "tuple[Union[str, bytes, Command[Any], shellous.Pipeline[Any]], ...]"
     "Command arguments including the program name as first argument."
 
     options: Options
     "Command options."
 
-    def __post_init__(self):
+    def __post_init__(self) -> None:
         "Validate the command."
         if len(self.args) == 0:
             raise ValueError("Command must include program name")
@@ -324,7 +412,7 @@ class Command:
             return f"...{name[-31:]}"
         return name
 
-    def stdin(self, input_: Any, *, close: bool = False) -> Self:
+    def stdin(self, input_: Any, *, close: bool = False) -> "Command[_RT]":
         "Pass `input` to command's standard input."
         new_options = self.options.set_stdin(input_, close)
         return Command(self.args, new_options)
@@ -335,9 +423,8 @@ class Command:
         *,
         append: bool = False,
         close: bool = False,
-    ) -> Self:
+    ) -> "Command[_RT]":
         "Redirect standard output to `output`."
-        _check_args(output, append)
         new_options = self.options.set_stdout(output, append, close)
         return Command(self.args, new_options)
 
@@ -347,13 +434,12 @@ class Command:
         *,
         append: bool = False,
         close: bool = False,
-    ) -> Self:
+    ) -> "Command[_RT]":
         "Redirect standard error to `error`."
-        _check_args(error, append)
         new_options = self.options.set_stderr(error, append, close)
         return Command(self.args, new_options)
 
-    def env(self, **kwds: str) -> Self:
+    def env(self, **kwds: Any) -> "Command[_RT]":
         """Return new command with augmented environment."""
         new_options = self.options.set_env(kwds)
         return Command(self.args, new_options)
@@ -361,10 +447,11 @@ class Command:
     def set(  # pylint: disable=unused-argument, too-many-locals
         self,
         *,
+        path: Unset[Optional[str]] = _UNSET,
         inherit_env: Unset[bool] = _UNSET,
-        encoding: Unset[Optional[str]] = _UNSET,
-        return_result: Unset[bool] = _UNSET,
-        catch_cancelled_error: Unset[bool] = _UNSET,
+        encoding: Unset[str] = _UNSET,
+        _return_result: Unset[bool] = _UNSET,
+        _catch_cancelled_error: Unset[bool] = _UNSET,
         exit_codes: Unset[Optional[Container[int]]] = _UNSET,
         timeout: Unset[Optional[float]] = _UNSET,
         cancel_timeout: Unset[float] = _UNSET,
@@ -372,13 +459,13 @@ class Command:
         alt_name: Unset[Optional[str]] = _UNSET,
         pass_fds: Unset[Iterable[int]] = _UNSET,
         pass_fds_close: Unset[bool] = _UNSET,
-        writable: Unset[bool] = _UNSET,
+        _writable: Unset[bool] = _UNSET,
         _start_new_session: Unset[bool] = _UNSET,
-        _preexec_fn: Unset[_Preexec_Fn_T] = _UNSET,
+        _preexec_fn: Unset[_PreexecFnT] = _UNSET,
         pty: Unset[PtyAdapterOrBool] = _UNSET,
         close_fds: Unset[bool] = _UNSET,
-        audit_callback: Unset[_Audit_Fn_T] = _UNSET,
-    ) -> Self:
+        audit_callback: Unset[_AuditFnT] = _UNSET,
+    ) -> "Command[_RT]":
         """Return new command with custom options set.
 
         **inherit_env** (bool) default=True<br>
@@ -388,21 +475,18 @@ class Command:
         environment is augmented/overriden by any variables specified in
         `Command.env`.
 
-        **encoding** (str | None) default="utf-8"<br>
-        String encoding to use for subprocess input/output. `encoding` of None
-        is deprecated. To specify `errors`, append it after a space. For
-        example, use "utf-8 replace" to specify "utf-8" with errors "replace".
+        **encoding** (str) default="utf-8"<br>
+        String encoding to use for subprocess input/output. To specify `errors`,
+        append it after a space. For example, use "utf-8 replace" to specify
+        "utf-8" with errors "replace".
 
-        **return_result** (bool) default=False<br>
+        **_return_result** (bool) default=False<br>
         When True, return a `Result` object instead of the standard output.
+        Private API -- use the `result` modifier instead.
 
-        **catch_cancelled_error** (bool) default=False<br>
-        When True, raise a `ResultError` when the command is cancelled. On the
-        plus side, this gives you access to the initial output of the command.
-        On the negative side, the `ResultError` swallows the `CancelledError`.
-        Your code may need to re-raise a CancelledError after dealing with the
-        partial result. When `catch_cancelled_error` is False, a cancelled command
-        will raise a `CancelledError`.
+        **_catch_cancelled_error** (bool) default=False<br>
+        When True, raise a `ResultError` when the command is cancelled.
+        Private API -- used internally by PipeRunner.
 
         **exit_codes** (set[int] | None) default=None<br>
         Set of allowed exit codes that will not raise a `ResultError`. By default,
@@ -438,14 +522,15 @@ class Command:
         Close the file descriptors in `pass_fds` immediately in the current
         process immediately after launching the subprocess.
 
-        **writable** (bool) default=False<br>
+        **_writable** (bool) default=False<br>
         Used to indicate process substitution is writing.
+        Private API -- use the `writable` modifier instead.
 
         **_start_new_session** (bool) default=False<br>
-        Provided for testing purposes only.
+        Private API -- provided for testing purposes only.
 
         **_preexec_fn** (Callable() | None) default=None<br>
-        Provided for testing purposes only.
+        Private API -- provided for testing purposes only.
 
         **pty** (bool | Callable(int)) default=False<br>
         If True, use a pseudo-terminal (pty) to control the child process.
@@ -494,14 +579,11 @@ class Command:
         """
         kwargs = locals()
         del kwargs["self"]
-        if encoding is None:
-            warn(
-                "encoding=None is deprecated; use encoding='latin1' instead",
-                DeprecationWarning,
-            )
+        if not encoding:
+            raise TypeError("invalid encoding")
         return Command(self.args, self.options.set(kwargs))
 
-    def _replace_args(self, new_args: list[Union[str, bytes]]) -> Self:
+    def _replace_args(self, new_args: Sequence[Any]) -> "Command[_RT]":
         """Return new command with arguments replaced by `new_args`.
 
         Arguments are NOT type-checked by the context. Program name must be the
@@ -511,50 +593,45 @@ class Command:
         assert new_args[0] is self.args[0]
         return Command(tuple(new_args), self.options)
 
-    def coro(self, *, _run_future: Optional[asyncio.Future[Runner]] = None):
+    def coro(
+        self,
+        *,
+        _run_future: Optional[asyncio.Future[Runner]] = None,
+    ) -> Coroutine[Any, Any, _RT]:
         "Return coroutine object to run awaitable."
-        return Runner.run_command(self, _run_future=_run_future)
+        return cast(
+            Coroutine[Any, Any, _RT],
+            Runner.run_command(self, _run_future=_run_future),
+        )
 
-    def run(self) -> Runner:
-        """Return a `Runner` to run the process incrementally.
-
-        ```
-        async with cmd.run() as run:
-            # do something with run.stdin, run.stdout, run.stderr...
-            # close run.stdin to signal we're done...
-        result = run.result()
-        ```
-        """
-        return Runner(self)
-
-    def __await__(self):
+    def __await__(self) -> "Generator[Any, None, _RT]":
         "Run process and return the standard output."
         return self.coro().__await__()
 
-    async def __aenter__(self):
+    async def __aenter__(self) -> Runner:
         "Enter the async context manager."
-        return await context_aenter(id(self), self.run())
+        return await context_aenter(id(self), Runner(self))
 
     async def __aexit__(
         self,
         exc_type: Optional[type[BaseException]],
         exc_value: Optional[BaseException],
         exc_tb: Optional[TracebackType],
-    ):
+    ) -> Optional[bool]:
         "Exit the async context manager."
         return await context_aexit(id(self), exc_type, exc_value, exc_tb)
 
-    def __aiter__(self):
+    def __aiter__(self) -> AsyncIterator[str]:
         "Return async iterator to iterate over output lines."
-        return self._readlines()
+        return aiter_preflight(self)._readlines()
 
-    async def _readlines(self):
+    async def _readlines(self) -> AsyncIterator[str]:
         "Async generator to iterate over lines."
-        async with self.run() as run:
+        async with Runner(self) as run:
             async for line in run:
                 yield line
 
-    def __call__(self, *args: Any) -> Self:
+    def __call__(self, *args: Any) -> "Command[_RT]":
         "Apply more arguments to the end of the command."
         if not args:
             return self
@@ -568,44 +645,53 @@ class Command:
         """
         return str(self.args[0])
 
-    def __or__(self, rhs: Any):
+    @overload
+    def __or__(self, rhs: StdoutType) -> "Command[_RT]":
+        ...  # pragma: no cover
+
+    @overload
+    def __or__(self, rhs: "Command[str]") -> "shellous.Pipeline[str]":
+        ...  # pragma: no cover
+
+    @overload
+    def __or__(
+        self,
+        rhs: "Command[shellous.Result]",
+    ) -> "shellous.Pipeline[shellous.Result]":
+        ...  # pragma: no cover
+
+    def __or__(self, rhs: Any) -> Any:
         "Bitwise or operator is used to build pipelines."
         if isinstance(rhs, STDOUT_TYPES):
             return self.stdout(rhs)
         return shellous.Pipeline.create(self) | rhs
 
-    def __ror__(self, lhs: Any):
+    def __ror__(self, lhs: StdinType) -> "Command[_RT]":
         "Bitwise or operator is used to build pipelines."
         if isinstance(lhs, STDIN_TYPES):
             return self.stdin(lhs)
         return NotImplemented
 
-    def __rshift__(self, rhs: Any):
+    def __rshift__(self, rhs: StdoutType) -> "Command[_RT]":
         "Right shift operator is used to build pipelines."
-        if isinstance(rhs, STDOUT_APPEND_TYPES):
+        if isinstance(
+            rhs, STDOUT_TYPES
+        ):  # pyright: ignore[reportUnnecessaryIsInstance]
             return self.stdout(rhs, append=True)
         return NotImplemented
 
-    def __mod__(self, rhs: "Command"):
-        "Modulo operator is used to concatenate commands."
-        if isinstance(rhs, Command):  # pyright: ignore[reportUnnecessaryIsInstance]
-            return self(rhs.args)
-        return NotImplemented
-
     @property
-    def writable(self) -> Self:
+    def writable(self) -> "Command[_RT]":
         "Set `writable` to True."
-        return self.set(writable=True)
+        return self.set(_writable=True)
 
     @property
-    def result(self) -> Self:
-        "Set `return_result` and `exit_codes`."
-        return self.set(return_result=True, exit_codes=range(-255, 256))
-
-
-def _check_args(out: Any, append: bool):
-    if append and not isinstance(out, STDOUT_APPEND_TYPES):
-        raise TypeError(f"{type(out)} does not support append")
+    def result(self) -> "Command[shellous.Result]":
+        "Set `_return_result` and `exit_codes`."
+        return cast(
+            Command[shellous.Result],
+            self.set(_return_result=True, exit_codes=range(-255, 256)),
+        )
 
 
 def coerce(args: Sequence[Any]) -> tuple[Any, ...]:
@@ -613,7 +699,8 @@ def coerce(args: Sequence[Any]) -> tuple[Any, ...]:
     result: list[Any] = []
     for arg in args:
         if isinstance(
-            arg, (str, bytes, bytearray, os.PathLike, Command, shellous.Pipeline)
+            arg,
+            (str, bytes, bytearray, os.PathLike, Command, shellous.Pipeline),
         ):
             result.append(arg)
         elif isinstance(arg, (list, tuple)):
