@@ -7,7 +7,7 @@ import errno
 import os
 import struct
 from dataclasses import dataclass
-from typing import Callable, NamedTuple, Optional, Union
+from typing import Any, Callable, NamedTuple, Optional, Union
 
 # The following modules are not supported on Windows.
 try:
@@ -19,14 +19,14 @@ except ImportError:  # pragma: no cover
     pass
 
 from .log import LOG_DETAIL, LOGGER, log_method
-from .util import BSD_DERIVED, close_fds
+from .util import BSD_DERIVED, BSD_FREEBSD, close_fds
 
 _STDOUT_FILENO = 1
 _LFLAG = 3
 _CC = 6
 
-# Either a function f(fd) that can set up a PTY descriptor, or a boolean.
-PtyAdapterOrBool = Union[Callable[[int], None], bool]
+PtyAdapter = Callable[[int], None]
+PtyAdapterOrBool = Union[PtyAdapter, bool]
 
 
 @dataclass
@@ -34,10 +34,10 @@ class ChildFd:
     "Make sure child fd is only closed once."
     child_fd: int
 
-    def __int__(self):
+    def __int__(self) -> int:
         return self.child_fd
 
-    def close(self):
+    def close(self) -> None:
         "Close the child file descriptor exactly once."
         if self.child_fd >= 0:
             fdesc = self.child_fd
@@ -54,7 +54,7 @@ class PtyFds(NamedTuple):
     writer: Optional[asyncio.StreamWriter] = None
 
     @log_method(LOG_DETAIL)
-    async def open_streams(self):
+    async def open_streams(self) -> "PtyFds":
         "Open pty reader/writer streams."
         reader, writer = await _open_pty_streams(self.parent_fd, self.child_fd)
         return PtyFds(
@@ -65,10 +65,10 @@ class PtyFds(NamedTuple):
             writer,
         )
 
-    def close(self):
+    def close(self) -> None:
         "Close pty file descriptors."
         if LOG_DETAIL:
-            LOGGER.info("PtyFds.close")
+            LOGGER.debug("PtyFds.close")
         self.child_fd.close()
         if self.writer:
             self.writer.close()
@@ -76,7 +76,7 @@ class PtyFds(NamedTuple):
             close_fds([self.parent_fd])
 
 
-def open_pty(pty_func: PtyAdapterOrBool):
+def open_pty(pty_func: PtyAdapterOrBool) -> PtyFds:
     "Open pseudo-terminal (pty) descriptors. Returns (PtyFds, child_fd)."
     parent_fd, child_fd = pty.openpty()
 
@@ -92,7 +92,7 @@ def open_pty(pty_func: PtyAdapterOrBool):
     )
 
 
-def set_ctty(ttypath: str):
+def set_ctty(ttypath: str) -> None:
     "Explicitly open the tty to make it become a controlling tty."
     # See https://github.com/python/cpython/blob/3.9/Lib/pty.py
 
@@ -104,42 +104,57 @@ def set_ctty(ttypath: str):
 class PtyStreamReaderProtocol(asyncio.StreamReaderProtocol):
     "Custom subclass of StreamReaderProtocol for pty's."
 
-    _pty_child_fd: Optional[ChildFd] = None
+    pty__child_fd: Optional[ChildFd] = None
+    pty__timer: Optional[asyncio.TimerHandle] = None  # Issue #378
 
-    def connection_lost(self, exc: Optional[Exception]):
+    def connection_lost(self, exc: Optional[Exception]) -> None:
         "Intercept EIO error and treat it as EOF."
         if LOG_DETAIL:
-            LOGGER.info("PtyStreamReaderProtocol.connection_lost ex=%r", exc)
+            LOGGER.debug("PtyStreamReaderProtocol.connection_lost ex=%r", exc)
+        if BSD_DERIVED and self.pty__timer is not None:
+            self.pty__timer.cancel()
         if isinstance(exc, OSError) and exc.errno == errno.EIO:
             exc = None
         super().connection_lost(exc)
 
     if BSD_DERIVED:
         # BSD only: On Linux, use the default behavior.
-        def data_received(self, data: bytes):
+
+        def _close_child_fd(self):
+            if self.pty__child_fd:
+                self.pty__child_fd.close()
+                self.pty__child_fd = None
+
+        if BSD_FREEBSD:
+            # Timer is only necessary on FreeBSD, not MacOS.
+            def connection_made(self, transport: asyncio.BaseTransport) -> None:
+                "Set up timer to close child fd when no data is received."
+                super().connection_made(transport)
+                if LOG_DETAIL:
+                    LOGGER.debug("PtyStreamReaderProtocol.connection_made")
+                self.pty__timer = self._loop.call_later(2.0, self._close_child_fd)  # type: ignore
+
+        def data_received(self, data: bytes) -> None:
             "Close child_fd when first data received."
-            if self._pty_child_fd:
-                self._pty_child_fd.close()
-                self._pty_child_fd = None
+            self._close_child_fd()
             return super().data_received(data)
 
-    def eof_received(self):
+    def eof_received(self) -> Optional[bool]:
         "Log when EOF received."
         if LOG_DETAIL:
-            LOGGER.info("PtyStreamReaderProtocol.eof_received")
+            LOGGER.debug("PtyStreamReaderProtocol.eof_received")
         return super().eof_received()
 
 
 async def _open_pty_streams(parent_fd: int, child_fd: ChildFd):
     "Open reader, writer streams for pty file descriptor."
-
     loop = asyncio.get_running_loop()
     reader = asyncio.StreamReader(loop=loop)
     reader_protocol = PtyStreamReaderProtocol(reader, loop=loop)
 
     # Stick reference to child_fd into protocol so we can close it after the
     # first data is received on BSD systems.
-    reader_protocol._pty_child_fd = child_fd  # pylint: disable=protected-access
+    reader_protocol.pty__child_fd = child_fd
 
     reader_transport, _ = await loop.connect_read_pipe(
         lambda: reader_protocol,
@@ -155,7 +170,7 @@ async def _open_pty_streams(parent_fd: int, child_fd: ChildFd):
     # Patch writer_transport.close so it also closes the reader_transport.
     def _close():
         if LOG_DETAIL:
-            LOGGER.info("writer_transport.close()")
+            LOGGER.debug("writer_transport.close()")
         _orig_close()
         reader_transport.close()
 
@@ -164,45 +179,44 @@ async def _open_pty_streams(parent_fd: int, child_fd: ChildFd):
     return reader, writer
 
 
-def raw(rows=0, cols=0):
+IntOrEllipsis = Union[int, type(...)]
+
+
+def raw(rows: IntOrEllipsis = 0, cols: IntOrEllipsis = 0) -> PtyAdapter:
     "Return a function that sets PtyOptions.child_fd to raw mode."
+    rows_int, cols_int = _inherit_term_size(rows, cols)
 
-    if Ellipsis in (rows, cols):
-        rows, cols = _inherit_term_size(rows, cols)
-
-    def _pty_set_raw(fdesc):
+    def _pty_set_raw(fdesc: int):
         tty.setraw(fdesc)
-        if rows or cols:
-            _set_term_size(fdesc, rows, cols)
+        if rows_int or cols_int:
+            _set_term_size(fdesc, rows_int, cols_int)
         assert _get_eof(fdesc) == b""
 
     return _pty_set_raw
 
 
-def cbreak(rows=0, cols=0):
+def cbreak(rows: IntOrEllipsis = 0, cols: IntOrEllipsis = 0) -> PtyAdapter:
     "Return a function that sets PtyOptions.child_fd to cbreak mode."
+    rows_int, cols_int = _inherit_term_size(rows, cols)
 
-    if Ellipsis in (rows, cols):
-        rows, cols = _inherit_term_size(rows, cols)
-
-    def _pty_set_cbreak(fdesc):
+    def _pty_set_cbreak(fdesc: int):
         tty.setcbreak(fdesc)
-        if rows or cols:
-            _set_term_size(fdesc, rows, cols)
+        if rows_int or cols_int:
+            _set_term_size(fdesc, rows_int, cols_int)
         assert _get_eof(fdesc) == b""
 
     return _pty_set_cbreak
 
 
-def cooked(rows=0, cols=0, echo=True):
+def cooked(
+    rows: IntOrEllipsis = 0, cols: IntOrEllipsis = 0, echo: bool = True
+) -> PtyAdapter:
     "Return a function that leaves PtyOptions.child_fd in cooked mode."
+    rows_int, cols_int = _inherit_term_size(rows, cols)
 
-    if Ellipsis in (rows, cols):
-        rows, cols = _inherit_term_size(rows, cols)
-
-    def _pty_set_canonical(fdesc):
-        if rows or cols:
-            _set_term_size(fdesc, rows, cols)
+    def _pty_set_canonical(fdesc: int):
+        if rows_int or cols_int:
+            _set_term_size(fdesc, rows_int, cols_int)
         if not echo:
             _set_term_echo(fdesc, False)
         assert _get_eof(fdesc) == b"\x04"
@@ -224,7 +238,7 @@ def _set_term_echo(fdesc: int, echo: bool):
         termios.tcsetattr(fdesc, termios.TCSADRAIN, attrs)
 
 
-def _set_term_size(fdesc, rows, cols):
+def _set_term_size(fdesc: int, rows: int, cols: int):
     "Set pseudo-terminal size."
     try:
         winsz = struct.pack("HHHH", rows, cols, 0, 0)
@@ -233,8 +247,11 @@ def _set_term_size(fdesc, rows, cols):
         LOGGER.warning("_set_term_size ex=%r", ex)
 
 
-def _inherit_term_size(rows, cols):
+def _inherit_term_size(rows: IntOrEllipsis, cols: IntOrEllipsis) -> tuple[int, int]:
     "Override ... with terminal setting from current stdin."
+    if Ellipsis not in (rows, cols):
+        return rows, cols  # type: ignore
+
     try:
         zeros = struct.pack("HHHH", 0, 0, 0, 0)
         winsz = fcntl.ioctl(_STDOUT_FILENO, tty.TIOCGWINSZ, zeros)  # pyright: ignore
@@ -248,12 +265,11 @@ def _inherit_term_size(rows, cols):
     if cols is Ellipsis:
         cols = winsz[1]
 
-    return rows, cols
+    return rows, cols  # type: ignore
 
 
 def _get_eof(fdesc: int):
     "Return the End-of-file character (EOF) if tty is in canonical mode only."
-
     eof = b""
     attrs = termios.tcgetattr(fdesc)
     if attrs[_LFLAG] & termios.ICANON:
@@ -278,7 +294,7 @@ def _patch_child_watcher():
 
     saved_add_handler = watcher.add_child_handler
 
-    def _add_child_handler(pid, callback, *args):
+    def _add_child_handler(pid: int, callback: Callable[..., object], *args: Any):
         if not _IGNORE_CHILD_PROCESS.get():
             saved_add_handler(pid, callback, *args)
 
