@@ -5,17 +5,22 @@ import enum
 import io
 from logging import Logger
 from pathlib import Path
-from typing import Any, Optional, Union
+from typing import TYPE_CHECKING, Any, Optional, TypeVar, Union
 
 from shellous.log import LOG_DETAIL, log_method
 from shellous.pty_util import PtyAdapterOrBool
 from shellous.util import decode
 
+if TYPE_CHECKING:
+    import shellous
+
 _CHUNK_SIZE = 8192
 _STDIN = 0
 _STDOUT = 1
 _STDERR = 2
 
+_CT = TypeVar("_CT", "shellous.Command[Any]", "shellous.Pipeline[Any]")
+
 
 class Redirect(enum.IntEnum):
     "Redirection constants."
@@ -24,20 +29,22 @@ class Redirect(enum.IntEnum):
     DEVNULL = asyncio.subprocess.DEVNULL  # -3
     CAPTURE = -10
     INHERIT = -11
-    RESULT = -12
+    BUFFER = -12
     DEFAULT = -20
 
-    def is_custom(self):
+    def is_custom(self) -> bool:
         "Return true if this redirect option is not built into asyncio."
         return self in {
             Redirect.CAPTURE,
             Redirect.INHERIT,
-            Redirect.RESULT,
+            Redirect.BUFFER,
             Redirect.DEFAULT,
         }
 
     @staticmethod
-    def from_default(obj: Any, fdesc: int, pty: PtyAdapterOrBool):
+    def from_default(
+        obj: Any, fdesc: int, pty: PtyAdapterOrBool
+    ) -> "Union[bytes, Redirect]":
         "Return object with Redirect.DEFAULT replaced by actual value."
         if not isinstance(obj, Redirect) or obj != Redirect.DEFAULT:
             return obj
@@ -45,23 +52,6 @@ class Redirect(enum.IntEnum):
         assert obj == Redirect.DEFAULT
         return _DEFAULT_REDIRECTION[(fdesc, bool(pty))]
 
-    @staticmethod
-    def from_literal(literal: Any, is_stderr: bool = False):
-        "Return object with literal values replaced by Redirect constant."
-
-        if isinstance(literal, (tuple, type(...), type(None))):
-            raise TypeError(f"literal {literal!r} is unsupported")
-
-        # For stderr, the literal `1` indicates that stderr is redirected to
-        # the same place as STDOUT.
-        if is_stderr and isinstance(literal, int) and literal == 1:
-            raise TypeError(f"stderr literal {literal!r} is unupported")
-
-        try:
-            return _LITERAL_REDIRECT.get(literal, literal)
-        except TypeError:
-            return literal
-
 
 # This table has the default redirections for (src, pty).
 # Sources are stdin, stdout, stderr. Used by Redirect.from_default().
@@ -69,18 +59,12 @@ _DEFAULT_REDIRECTION: dict[tuple[int, bool], Union[bytes, Redirect]] = {
     # (FD, PTY)
     (_STDIN, False): b"",
     (_STDIN, True): Redirect.CAPTURE,
-    (_STDOUT, False): Redirect.CAPTURE,
-    (_STDOUT, True): Redirect.CAPTURE,
-    (_STDERR, False): Redirect.RESULT,
+    (_STDOUT, False): Redirect.BUFFER,
+    (_STDOUT, True): Redirect.BUFFER,
+    (_STDERR, False): Redirect.BUFFER,
     (_STDERR, True): Redirect.STDOUT,
 }
 
-_LITERAL_REDIRECT: dict[Any, Redirect] = {
-    None: Redirect.DEVNULL,
-    ...: Redirect.INHERIT,
-    (): Redirect.CAPTURE,
-}
-
 # Used in Command and Pipeline to implement operator overloading.
 STDIN_TYPES = (
     str,
@@ -93,6 +77,17 @@ STDIN_TYPES = (
     asyncio.StreamReader,
 )
 
+StdinType = Union[
+    str,
+    bytes,
+    Path,
+    bytearray,
+    io.IOBase,
+    int,
+    Redirect,
+    asyncio.StreamReader,
+]
+
 STDOUT_TYPES = (
     Path,
     bytearray,
@@ -103,7 +98,15 @@ STDOUT_TYPES = (
     asyncio.StreamWriter,
 )
 
-STDOUT_APPEND_TYPES = (Path,)
+StdoutType = Union[
+    Path,
+    bytearray,
+    io.IOBase,
+    int,
+    Redirect,
+    Logger,
+    asyncio.StreamWriter,
+]
 
 
 async def _drain(stream: asyncio.StreamWriter):
@@ -158,7 +161,6 @@ async def write_reader(
     eof: Optional[bytes] = None,
 ):
     "Copy from reader to writer."
-
     ends_with_newline = True
     try:
         while True:
@@ -275,12 +277,29 @@ async def copy_streamwriter(source: asyncio.StreamReader, dest: asyncio.StreamWr
 
 
 @log_method(LOG_DETAIL)
-async def read_lines(source: asyncio.StreamReader, encoding: Optional[str]):
+async def read_lines(source: asyncio.StreamReader, encoding: str):
     "Async iterator over lines in stream."
-    if encoding is None:
-        async for line in source:
-            yield line
+    async for line in source:
+        yield decode(line, encoding)
 
+
+def aiter_preflight(cmd: _CT) -> _CT:
+    "Fix up command or pipeline when iterating using __aiter__."
+    if cmd.options.output == Redirect.DEFAULT:
+        result = cmd.stdout(Redirect.CAPTURE)
+    elif (
+        cmd.options.output == Redirect.DEVNULL and cmd.options.error == Redirect.STDOUT
+    ):
+        result = cmd.stderr(Redirect.CAPTURE)
     else:
-        async for line in source:
-            yield decode(line, encoding)
+        result = cmd
+
+    # Check for ambiguous capture situation. This can lead to deadlocking the
+    # process being run.
+    if (
+        result.options.output == Redirect.CAPTURE
+        and result.options.error == Redirect.CAPTURE
+    ):
+        raise RuntimeError("multiple capture not supported in iterator")
+
+    return result
