@@ -7,7 +7,7 @@ import sys
 from logging import Logger
 from pathlib import Path
 from types import TracebackType
-from typing import Any, Optional, Union, cast
+from typing import Any, AsyncIterator, Coroutine, Optional, TextIO, TypeVar, Union, cast
 
 import shellous
 import shellous.redirect as redir
@@ -15,7 +15,12 @@ from shellous import pty_util
 from shellous.harvest import harvest, harvest_results
 from shellous.log import LOG_DETAIL, LOG_ENTER, LOG_EXIT, LOGGER, log_method, log_timer
 from shellous.redirect import Redirect
-from shellous.result import RESULT_STDERR_LIMIT, Result, make_result
+from shellous.result import (
+    RESULT_STDERR_LIMIT,
+    Result,
+    check_result,
+    convert_result_list,
+)
 from shellous.util import (
     BSD_DERIVED,
     SupportsClose,
@@ -23,13 +28,13 @@ from shellous.util import (
     poll_wait_pid,
     uninterrupted,
     verify_dev_fd,
-    which,
 )
 
+# pyright: reportPrivateUsage=false
+
 _KILL_TIMEOUT = 3.0
 _CLOSE_TIMEOUT = 0.25
 _UNKNOWN_EXIT_CODE = 255
-_UNLAUNCHED_EXIT_CODE = -255
 
 AUDIT_EVENT_SUBPROCESS_SPAWN = "byllyfish/shellous.subprocess_spawn"
 """Audit event raised by sys.audit() when shellous runs a subprocess.
@@ -37,23 +42,28 @@ AUDIT_EVENT_SUBPROCESS_SPAWN = "byllyfish/shellous.subprocess_spawn"
 The audit event has one argument: the name of the command.
 """
 
+UNLAUNCHED_EXIT_CODE = -255
+"""Special exit code used in audit events when process launch itself
+was cancelled."""
+
+
+_T = TypeVar("_T")
+
 
 def _is_cancelled(ex: BaseException):
     return isinstance(ex, asyncio.CancelledError)
 
 
-def _is_writable(cmd: "Union[shellous.Command, shellous.Pipeline]"):
-    "Return true if command/pipeline has `writable` set."
+def _is_writable(cmd: "Union[shellous.Command[Any], shellous.Pipeline[Any]]"):
+    "Return true if command/pipeline has `_writable` set."
     if isinstance(cmd, shellous.Pipeline):
         # Pipelines need to check both the last/first commands.
-        return cmd.options.writable or cmd[0].options.writable
-    return cmd.options.writable
+        return cmd.options._writable or cmd[0].options._writable
+    return cmd.options._writable
 
 
-def _enc(encoding: Optional[str]):
+def _enc(encoding: str) -> list[str]:
     "Helper function to split the encoding name into codec and errors."
-    if encoding is None:
-        raise TypeError("when encoding is None, input must be bytes")
     return encoding.split(maxsplit=1)
 
 
@@ -70,17 +80,19 @@ class _RunOptions:
     ```
     """
 
-    command: "shellous.Command"
-    encoding: Optional[str]
+    command: "shellous.Command[Any]"
+    encoding: str
     open_fds: list[Union[int, SupportsClose]]
     input_bytes: Optional[bytes]
     pos_args: list[Union[str, bytes]]
     kwd_args: dict[str, Any]
-    subcmds: "list[Union[shellous.Command, shellous.Pipeline]]"
+    subcmds: "list[Union[shellous.Command[Any], shellous.Pipeline[Any]]]"
     pty_fds: Optional[pty_util.PtyFds]
+    output_bytes: Optional[bytearray]
     error_bytes: Optional[bytearray]
+    is_stderr_only: bool = False
 
-    def __init__(self, command: "shellous.Command"):
+    def __init__(self, command: "shellous.Command[Any]"):
         self.command = command
         self.encoding = command.options.encoding
         self.open_fds = []
@@ -89,6 +101,7 @@ class _RunOptions:
         self.kwd_args = {}
         self.subcmds = []
         self.pty_fds = None
+        self.output_bytes = None
         self.error_bytes = None
 
     def __enter__(self):
@@ -106,8 +119,12 @@ class _RunOptions:
 
             # If executable does not include an absolute/relative directory,
             # resolve it using PATH.
-            if not os.path.dirname(self.pos_args[0]):
-                self.pos_args[0] = which(self.pos_args[0])
+            executable = self.pos_args[0]
+            if not os.path.dirname(executable):  # type: ignore (path)
+                found_executable = _cmd.options.which(executable)
+                if found_executable is None:
+                    raise FileNotFoundError(executable)
+                self.pos_args[0] = found_executable
 
         except Exception as ex:
             if LOG_DETAIL:
@@ -142,7 +159,7 @@ class _RunOptions:
         Returns new command line arguments with /dev/fd path substitutions.
         """
         if sys.platform == "win32":
-            raise RuntimeError("process substitution not supported on Windows")
+            raise RuntimeError("process substitution not supported")  # pragma: no cover
 
         new_args: list[Union[str, bytes]] = []
         new_fds: list[int] = []
@@ -185,6 +202,10 @@ class _RunOptions:
             self.encoding,
         )
 
+        # Handle "stderr only" case when stdout -> DEVNULL, stderr -> STDOUT.
+        if options.output == Redirect.DEVNULL and options.error == Redirect.STDOUT:
+            self.is_stderr_only = True
+
         stdout = self._setup_output(
             Redirect.from_default(options.output, 1, options.pty),
             options.output_append,
@@ -200,8 +221,8 @@ class _RunOptions:
         )
 
         # Set up PTY here. This is the first half. Second half in `Runner`.
-        start_session = options._start_new_session  # pylint: disable=protected-access
-        preexec_fn = options._preexec_fn  # pylint: disable=protected-access
+        start_session = options._start_new_session
+        preexec_fn = options._preexec_fn
 
         if options.pty:
             assert preexec_fn is None
@@ -240,7 +261,7 @@ class _RunOptions:
             # Only set close_fds if it is not already set.
             self.kwd_args["close_fds"] = options.close_fds
 
-    def _setup_input(self, input_, close, encoding):
+    def _setup_input(self, input_: Any, close: bool, encoding: str):
         "Set up process input."
         assert input_ is not None
 
@@ -256,7 +277,7 @@ class _RunOptions:
             # Custom support for Redirect constants.
             if input_ == Redirect.INHERIT:
                 stdin = sys.stdin
-            elif input_ == Redirect.RESULT:
+            elif input_ == Redirect.BUFFER:
                 raise TypeError(f"unsupported input type: {input_!r}")
             else:
                 # CAPTURE uses stdin == PIPE.
@@ -277,26 +298,28 @@ class _RunOptions:
 
         return stdin, input_bytes
 
-    def _setup_output(self, output, append, close, sys_stream):
+    def _setup_output(self, output: Any, append: bool, close: bool, sys_stream: TextIO):
         "Set up process output. Used for both stdout and stderr."
         assert output is not None
 
         stdout = asyncio.subprocess.PIPE
 
-        if isinstance(output, (str, bytes, os.PathLike)):
-            if isinstance(output, (str, bytes)):  # TODO: remove this check...
-                raise TypeError("Using str|bytes as output file is unsupported")
+        if isinstance(output, os.PathLike):
             mode = "ab" if append else "wb"
-            stdout = open(output, mode=mode)  # pylint: disable=consider-using-with
+            stdout = open(cast(os.PathLike[str], output), mode=mode)
             self.open_fds.append(stdout)
+        elif self.is_stderr_only and sys_stream == sys.stderr:
+            assert output == Redirect.STDOUT
+            self.output_bytes = bytearray()
+            assert stdout == asyncio.subprocess.PIPE
         elif isinstance(output, Redirect) and output.is_custom():
             # Custom support for Redirect constants.
-            if output == Redirect.RESULT:
-                # We do not support redirecting stdout to RESULT (only stderr).
+            if output == Redirect.BUFFER:
                 if sys_stream == sys.stdout:
-                    raise TypeError(f"unsupported output type: {output!r}")
+                    self.output_bytes = bytearray()
+                else:
+                    self.error_bytes = bytearray()
                 assert stdout == asyncio.subprocess.PIPE
-                self.error_bytes = bytearray()
             elif output == Redirect.INHERIT:
                 stdout = sys_stream
             else:
@@ -312,21 +335,28 @@ class _RunOptions:
             output, (io.StringIO, io.BytesIO, bytearray, Logger, asyncio.StreamWriter)
         ):
             # Shellous-supported output classes.
+            _set_position(output, append)
             assert stdout == asyncio.subprocess.PIPE
         elif isinstance(output, io.IOBase):
             # Client-managed File-like object.
+            _set_position(output, append)
             stdout = output
         else:
             raise TypeError(f"unsupported output type: {output!r}")
 
         return stdout
 
-    def _setup_pty1(self, stdin, stdout, stderr, pty):
+    def _setup_pty1(
+        self,
+        stdin: Any,
+        stdout: Any,
+        stderr: Any,
+        pty: pty_util.PtyAdapterOrBool,
+    ):
         """Set up pseudo-terminal and return (stdin, stdout, stderr, preexec_fn).
 
         Initializes `self.pty_fds`.
         """
-
         self.pty_fds = pty_util.open_pty(pty)
 
         # On BSD-derived systems like FreeBSD and Darwin, we delay closing the
@@ -337,7 +367,7 @@ class _RunOptions:
             self.open_fds.append(self.pty_fds.child_fd)
 
         if LOG_DETAIL:
-            LOGGER.info("_setup_pty1: %r", self.pty_fds)
+            LOGGER.debug("_setup_pty1: %r", self.pty_fds)
 
         child_fd = int(self.pty_fds.child_fd)
 
@@ -364,48 +394,51 @@ class Runner:
     ```
     """
 
-    stdin = None
+    stdin: Optional[asyncio.StreamWriter] = None
     "Process standard input."
 
-    stdout = None
+    stdout: Optional[asyncio.StreamReader] = None
     "Process standard output."
 
-    stderr = None
+    stderr: Optional[asyncio.StreamReader] = None
     "Process standard error."
 
-    def __init__(self, command):
+    _options: _RunOptions
+    _tasks: list[asyncio.Task[Any]]
+    _proc: Optional["asyncio.subprocess.Process"] = None
+    _cancelled: bool = False
+    _timer: Optional[asyncio.TimerHandle] = None
+    _timed_out: bool = False
+    _last_signal: Optional[int] = None
+
+    def __init__(self, command: "shellous.Command[Any]"):
         self._options = _RunOptions(command)
-        self._cancelled = False
-        self._proc = None
         self._tasks = []
-        self._timer = None  # asyncio.TimerHandle
-        self._timed_out = False  # True if runner timeout expired
-        self._last_signal = None
 
     @property
-    def name(self):
+    def name(self) -> str:
         "Return name of process being run."
         return self.command.name
 
     @property
-    def command(self):
+    def command(self) -> "shellous.Command[Any]":
         "Return the command being run."
         return self._options.command
 
     @property
-    def pid(self):
+    def pid(self) -> Optional[int]:
         "Return the command's process ID."
         if not self._proc:
             return None
         return self._proc.pid
 
     @property
-    def returncode(self):
+    def returncode(self) -> Optional[int]:
         "Process's exit code."
         if not self._proc:
             if self._cancelled:
                 # The process was cancelled before starting.
-                return _UNLAUNCHED_EXIT_CODE
+                return UNLAUNCHED_EXIT_CODE
             return None
         code = self._proc.returncode
         if code == _UNKNOWN_EXIT_CODE and self._last_signal is not None:
@@ -415,49 +448,57 @@ class Runner:
         return code
 
     @property
-    def cancelled(self):
+    def cancelled(self) -> bool:
         "Return True if the command was cancelled."
         return self._cancelled
 
-    def result(self, output_bytes=b""):
+    def result(self) -> Result:
         "Check process exit code and raise a ResultError if necessary."
         code = self.returncode
         assert code is not None
 
         result = Result(
             exit_code=code,
-            output_bytes=output_bytes,
+            output_bytes=bytes(self._options.output_bytes or b""),
             error_bytes=bytes(self._options.error_bytes or b""),
             cancelled=self._cancelled,
             encoding=self._options.encoding,
         )
 
-        # TODO: This method should return a Result object... (not str|bytes)
-        return make_result(self.command, result, self._cancelled, self._timed_out)
+        return check_result(
+            result,
+            self.command.options,
+            self._cancelled,
+            self._timed_out,
+        )
 
-    def add_task(self, coro, tag=""):
+    def add_task(
+        self,
+        coro: Coroutine[Any, Any, _T],
+        tag: str = "",
+    ) -> asyncio.Task[_T]:
         "Add a background task."
         task_name = f"{self.name}#{tag}"
         task = asyncio.create_task(coro, name=task_name)
         self._tasks.append(task)
         return task
 
-    def send_signal(self, sig: int):
+    def send_signal(self, sig: int) -> None:
         "Send an arbitrary signal to the process if it is running."
         if self.returncode is None:
             self._signal(sig)
 
-    def cancel(self):
+    def cancel(self) -> None:
         "Cancel the running process if it is running."
         if self.returncode is None:
             self._signal(self.command.options.cancel_signal)
 
-    def _is_bsd_pty(self):
+    def _is_bsd_pty(self) -> bool:
         "Return true if we're running a pty on BSD."
-        return BSD_DERIVED and self._options.pty_fds
+        return BSD_DERIVED and bool(self._options.pty_fds)
 
     @log_method(LOG_DETAIL)
-    async def _wait(self):
+    async def _wait(self) -> None:
         "Normal wait for background I/O tasks and process to finish."
         assert self._proc
 
@@ -468,13 +509,13 @@ class Runner:
                 await self._waiter()
 
         except asyncio.CancelledError:
-            LOGGER.info("Runner.wait cancelled %r", self)
+            LOGGER.debug("Runner.wait cancelled %r", self)
             self._set_cancelled()
             self._tasks.clear()  # all tasks were cancelled
             await self._kill()
 
         except Exception as ex:
-            LOGGER.info("Runner.wait exited with error %r ex=%r", self, ex)
+            LOGGER.debug("Runner.wait exited with error %r ex=%r", self, ex)
             self._tasks.clear()  # all tasks were cancelled
             await self._kill()
             raise  # re-raise exception
@@ -525,7 +566,7 @@ class Runner:
         "Send a signal to the process."
         assert self._proc is not None  # (pyright)
 
-        LOGGER.info("Runner.signal %r signal=%r", self, sig)
+        LOGGER.debug("Runner.signal %r signal=%r", self, sig)
         self._audit_callback("signal", signal=sig)
 
         if sig is None:
@@ -554,7 +595,7 @@ class Runner:
                 LOGGER.error("%r failed to kill process %r", self, self._proc)
                 raise RuntimeError(f"Unable to kill process {self._proc!r}") from ex
 
-    @log_method(LOG_ENTER, _info=True)
+    @log_method(LOG_ENTER)
     async def __aenter__(self):
         "Set up redirections and launch subprocess."
         self._audit_callback("start")
@@ -565,22 +606,22 @@ class Runner:
             self._audit_callback("stop", failure=type(ex).__name__)
             raise
         finally:
-            if self._cancelled and self.command.options.catch_cancelled_error:
+            if self._cancelled and self.command.options._catch_cancelled_error:
                 # Raises ResultError instead of CancelledError.
                 self.result()
 
     @log_method(LOG_DETAIL)
     async def _start(self):
         "Set up redirections and launch subprocess."
-        assert self._proc is None
+        # assert self._proc is None
         assert not self._tasks
 
         try:
             # Set up subprocess arguments and launch subprocess.
             with self._options as opts:
                 await self._subprocess_spawn(opts)
-                assert self._proc is not None  # (pyright)
 
+            assert self._proc is not None
             stdin = self._proc.stdin
             stdout = self._proc.stdout
             stderr = self._proc.stderr
@@ -595,6 +636,10 @@ class Runner:
                 if opts.error_bytes is not None:
                     error = opts.error_bytes
                     limit = RESULT_STDERR_LIMIT
+                elif opts.is_stderr_only:
+                    assert stdout is None
+                    assert opts.output_bytes is not None
+                    error = opts.output_bytes
                 else:
                     error = opts.command.options.error
                 stderr = self._setup_output_sink(
@@ -602,16 +647,20 @@ class Runner:
                 )
 
             if stdout is not None:
-                output = opts.command.options.output
+                limit = -1
+                if opts.output_bytes is not None:
+                    output = opts.output_bytes
+                else:
+                    output = opts.command.options.output
                 stdout = self._setup_output_sink(
-                    stdout, output, opts.encoding, "stdout"
+                    stdout, output, opts.encoding, "stdout", limit
                 )
 
             if stdin is not None:
                 stdin = self._setup_input_source(stdin, opts)
 
         except (Exception, asyncio.CancelledError) as ex:
-            LOGGER.info("Runner._start %r ex=%r", self, ex)
+            LOGGER.debug("Runner._start %r ex=%r", self, ex)
             if _is_cancelled(ex):
                 self._set_cancelled()
             if self._proc:
@@ -634,8 +683,9 @@ class Runner:
         return self
 
     @log_method(LOG_DETAIL)
-    async def _subprocess_spawn(self, opts):
+    async def _subprocess_spawn(self, opts: _RunOptions):
         "Start the subprocess."
+        assert self._proc is None
 
         # Second half of pty setup.
         if opts.pty_fds:
@@ -685,7 +735,7 @@ class Runner:
         self._cancelled = True
         self._stop_timer()
 
-    def _start_timer(self, timeout):
+    def _start_timer(self, timeout: Optional[float]):
         "Start an optional timer to cancel the process if `timeout` desired."
         assert self._timer is None
         if timeout is not None:
@@ -696,7 +746,7 @@ class Runner:
                 asyncio.current_task(),
             )
 
-    def _set_timer_expired(self, main_task):
+    def _set_timer_expired(self, main_task: asyncio.Task[Any]):
         "Set a flag when the timer expires and cancel the main task."
         self._timed_out = True
         self._timer = None
@@ -707,7 +757,11 @@ class Runner:
             self._timer.cancel()
             self._timer = None
 
-    def _setup_input_source(self, stream, opts):
+    def _setup_input_source(
+        self,
+        stream: asyncio.StreamWriter,
+        opts: _RunOptions,
+    ):
         "Set up a task to read from custom input source."
         tag = "stdin"
         eof = opts.pty_fds.eof if opts.pty_fds else None
@@ -733,39 +787,54 @@ class Runner:
 
         return stream
 
-    def _setup_output_sink(self, stream, sink, encoding, tag, limit=-1):
+    def _setup_output_sink(
+        self,
+        stream: asyncio.StreamReader,
+        sink: Any,
+        encoding: str,
+        tag: str,
+        limit: int = -1,
+    ) -> Optional[asyncio.StreamReader]:
         "Set up a task to write to custom output sink."
         if isinstance(sink, io.StringIO):
-            if encoding is None:
-                raise TypeError("StringIO not supported when encoding=None")
             self.add_task(redir.copy_stringio(stream, sink, encoding), tag)
-            stream = None
-        elif isinstance(sink, io.BytesIO):
+            return None
+
+        if isinstance(sink, io.BytesIO):
             self.add_task(redir.copy_bytesio(stream, sink), tag)
-            stream = None
-        elif isinstance(sink, bytearray):
+            return None
+
+        if isinstance(sink, bytearray):
             # N.B. `limit` is only supported for bytearray.
             if limit >= 0:
                 self.add_task(redir.copy_bytearray_limit(stream, sink, limit), tag)
             else:
                 self.add_task(redir.copy_bytearray(stream, sink), tag)
-            stream = None
-        elif isinstance(sink, Logger):
+            return None
+
+        if isinstance(sink, Logger):
             self.add_task(redir.copy_logger(stream, sink, encoding), tag)
-            stream = None
-        elif isinstance(sink, asyncio.StreamWriter):
+            return None
+
+        if isinstance(sink, asyncio.StreamWriter):
             self.add_task(redir.copy_streamwriter(stream, sink), tag)
-            stream = None
+            return None
+
         return stream
 
-    @log_method(LOG_EXIT, exc_value=2)
-    async def __aexit__(self, _exc_type, exc_value, _exc_tb):
+    @log_method(LOG_EXIT)
+    async def __aexit__(
+        self,
+        _exc_type: Union[type[BaseException], None],
+        exc_value: Union[BaseException, None],
+        _exc_tb: Optional[TracebackType],
+    ):
         "Wait for process to exit and handle cancellation."
         suppress = False
         try:
             suppress = await self._finish(exc_value)
         except asyncio.CancelledError:
-            LOGGER.info("Runner cancelled inside _finish %r", self)
+            LOGGER.debug("Runner cancelled inside _finish %r", self)
             self._set_cancelled()
         finally:
             self._stop_timer()  # failsafe just in case
@@ -774,13 +843,13 @@ class Runner:
         if (
             self._cancelled
             and self._timed_out
-            and not self.command.options.catch_cancelled_error
+            and not self.command.options._catch_cancelled_error
         ):
             raise asyncio.TimeoutError()
         return suppress
 
     @log_method(LOG_DETAIL)
-    async def _finish(self, exc_value):
+    async def _finish(self, exc_value: Union[BaseException, None]):
         "Finish the run. Return True only if `exc_value` should be suppressed."
         assert self._proc
 
@@ -806,7 +875,7 @@ class Runner:
             self._options.pty_fds.close()
 
         # Make sure the transport is closed (for asyncio and uvloop).
-        self._proc._transport.close()  # pyright: ignore pylint: disable=protected-access
+        self._proc._transport.close()  # pyright: ignore
 
         # _close can be called when unwinding exceptions. We need to handle
         # the case that the process has not exited yet.
@@ -829,18 +898,25 @@ class Runner:
         except asyncio.TimeoutError:
             LOGGER.critical("Runner._close %r timeout stdin=%r", self, self._proc.stdin)
 
-    def _audit_callback(self, phase, *, failure=None, signal=None):
+    def _audit_callback(
+        self,
+        phase: str,
+        *,
+        failure: str = "",
+        signal: Optional[int] = None,
+    ):
         "Call `audit_callback` if there is one."
         callback = self.command.options.audit_callback
         if callback:
-            info: dict[str, Any] = {"runner": self}
-            if failure:
-                info["failure"] = failure
-            if phase == "signal":
-                info["signal"] = _signame(signal)
+            sig = _signame(signal) if phase == "signal" else ""
+            info: shellous.AuditEventInfo = {
+                "runner": self,
+                "failure": failure,
+                "signal": sig,
+            }
             callback(phase, info)
 
-    def __repr__(self):
+    def __repr__(self) -> str:
         "Return string representation of Runner."
         cancelled = " cancelled" if self._cancelled else ""
         if self._proc:
@@ -851,44 +927,36 @@ class Runner:
 
     async def _readlines(self):
         "Iterate over lines in stdout/stderr"
-        if self.stdin or (self.stdout and self.stderr):
-            raise RuntimeError("multiple capture not supported in iterator")
-
         stream = self.stdout or self.stderr
         if stream:
             async for line in redir.read_lines(stream, self._options.encoding):
                 yield line
 
-    def __aiter__(self):
+    def __aiter__(self) -> AsyncIterator[str]:
         "Return asynchronous iterator over stdout/stderr."
         return self._readlines()
 
     @staticmethod
     async def run_command(
-        command: "shellous.Command",
+        command: "shellous.Command[Any]",
         *,
         _run_future: Optional[asyncio.Future["Runner"]] = None,
-    ):
+    ) -> Union[str, Result]:
         "Run a command. This is the main entry point for Runner."
         if not _run_future and _is_multiple_capture(command):
             LOGGER.warning("run_command: multiple capture requires 'async with'")
             _cleanup(command)
             raise ValueError("multiple capture requires 'async with'")
 
-        output_bytes = bytearray()
-
-        async with command.run() as run:
+        async with Runner(command) as run:
             if _run_future is not None:
                 # Return streams to caller in another task.
                 _run_future.set_result(run)
 
-            else:
-                # Read the output here and return it.
-                stream = run.stdout or run.stderr
-                if stream:
-                    await redir.copy_bytearray(stream, output_bytes)
-
-        return run.result(bytes(output_bytes))
+        result = run.result()
+        if command.options._return_result:
+            return result
+        return result.output
 
 
 class PipeRunner:
@@ -901,16 +969,23 @@ class PipeRunner:
     ```
     """
 
-    stdin = None
+    stdin: Optional[asyncio.StreamWriter] = None
     "Pipeline standard input."
 
-    stdout = None
+    stdout: Optional[asyncio.StreamReader] = None
     "Pipeline standard output."
 
-    stderr = None
+    stderr: Optional[asyncio.StreamReader] = None
     "Pipeline standard error."
 
-    def __init__(self, pipe, *, capturing):
+    _pipe: "shellous.Pipeline[Any]"
+    _capturing: bool
+    _tasks: list[asyncio.Task[Any]]
+    _encoding: str
+    _cancelled: bool = False
+    _results: Optional[list[Union[BaseException, Result]]] = None
+
+    def __init__(self, pipe: "shellous.Pipeline[Any]", *, capturing: bool):
         """`capturing=True` indicates we are within an `async with` block and
         client needs to access `stdin` and `stderr` streams.
         """
@@ -919,20 +994,29 @@ class PipeRunner:
         self._pipe = pipe
         self._cancelled = False
         self._tasks = []
-        self._results = None
         self._capturing = capturing
         self._encoding = pipe.options.encoding
 
     @property
-    def name(self):
+    def name(self) -> str:
         "Return name of the pipeline."
         return self._pipe.name
 
-    def result(self):
+    def result(self) -> Result:
         "Return `Result` object for PipeRunner."
-        return make_result(self._pipe, self._results, self._cancelled)
+        assert self._results is not None
 
-    def add_task(self, coro, tag=""):
+        return check_result(
+            convert_result_list(self._results, self._cancelled),
+            self._pipe.options,
+            self._cancelled,
+        )
+
+    def add_task(
+        self,
+        coro: Coroutine[Any, Any, _T],
+        tag: str = "",
+    ) -> asyncio.Task[_T]:
         "Add a background task."
         task_name = f"{self.name}#{tag}"
         task = asyncio.create_task(coro, name=task_name)
@@ -940,12 +1024,12 @@ class PipeRunner:
         return task
 
     @log_method(LOG_DETAIL)
-    async def _wait(self, *, kill=False):
+    async def _wait(self, *, kill: bool = False):
         "Wait for pipeline to finish."
         assert self._results is None
 
         if kill:
-            LOGGER.info("PipeRunner.wait killing pipe %r", self)
+            LOGGER.debug("PipeRunner.wait killing pipe %r", self)
             for task in self._tasks:
                 task.cancel()
 
@@ -966,8 +1050,13 @@ class PipeRunner:
             await self._wait(kill=True)
             raise
 
-    @log_method(LOG_EXIT, exc_value=2)
-    async def __aexit__(self, _exc_type, exc_value, _exc_tb):
+    @log_method(LOG_EXIT)
+    async def __aexit__(
+        self,
+        _exc_type: Union[type[BaseException], None],
+        exc_value: Union[BaseException, None],
+        _exc_tb: Optional[TracebackType],
+    ):
         "Wait for pipeline to exit and handle cancellation."
         suppress = False
         try:
@@ -978,7 +1067,7 @@ class PipeRunner:
         return suppress
 
     @log_method(LOG_DETAIL)
-    async def _finish(self, exc_value):
+    async def _finish(self, exc_value: Optional[BaseException]) -> bool:
         "Wait for pipeline to exit and handle cancellation."
         if exc_value is not None:
             LOGGER.warning("PipeRunner._finish exc_value=%r", exc_value)
@@ -988,6 +1077,7 @@ class PipeRunner:
             return self._cancelled
 
         await self._wait()
+        return False
 
     @log_method(LOG_DETAIL)
     async def _start(self):
@@ -1018,7 +1108,7 @@ class PipeRunner:
             close_fds(open_fds)
             raise
 
-    def _setup_pipeline(self, open_fds):
+    def _setup_pipeline(self, open_fds: list[int]):
         """Return the pipeline stitched together with pipe fd's.
 
         Each created open file descriptor is added to `open_fds` so it can
@@ -1035,14 +1125,13 @@ class PipeRunner:
             cmds[i + 1] = cmds[i + 1].stdin(read_fd, close=True)
 
         for i in range(cmd_count):
-            cmds[i] = cmds[i].set(return_result=True, catch_cancelled_error=True)
+            cmds[i] = cmds[i].set(_return_result=True, _catch_cancelled_error=True)
 
         return cmds
 
     @log_method(LOG_DETAIL)
-    async def _setup_capturing(self, cmds):
+    async def _setup_capturing(self, cmds: "list[shellous.Command[Any]]"):
         """Set up capturing and return (stdin, stdout, stderr) streams."""
-
         loop = asyncio.get_event_loop()
         first_fut = loop.create_future()
         last_fut = loop.create_future()
@@ -1069,7 +1158,7 @@ class PipeRunner:
 
         return (stdin, stdout, stderr)
 
-    def __repr__(self):
+    def __repr__(self) -> str:
         "Return string representation of PipeRunner."
         cancelled_info = ""
         if self._cancelled:
@@ -1079,53 +1168,53 @@ class PipeRunner:
             result_info = f" results={self._results!r}"
         return f"<PipeRunner {self.name!r}{cancelled_info}{result_info}>"
 
-    async def _readlines(self):
+    async def _readlines(self) -> AsyncIterator[str]:
         "Iterate over lines in stdout/stderr"
-        if self.stdin or (self.stdout and self.stderr):
-            raise RuntimeError("multiple capture not supported in iterator")
-
         stream = self.stdout or self.stderr
         if stream:
             async for line in redir.read_lines(stream, self._encoding):
                 yield line
 
-    def __aiter__(self):
+    def __aiter__(self) -> AsyncIterator[str]:
         "Return asynchronous iterator over stdout/stderr."
         return self._readlines()
 
     @staticmethod
-    async def run_pipeline(pipe: "shellous.Pipeline"):
+    async def run_pipeline(pipe: "shellous.Pipeline[Any]") -> Union[str, Result]:
         "Run a pipeline. This is the main entry point for PipeRunner."
-
         run = PipeRunner(pipe, capturing=False)
         async with run:
             pass
-        return run.result()
+
+        result = run.result()
+        if pipe.options._return_result:
+            return result
+        return result.output
 
 
-def _uses_process_substitution(cmd: "shellous.Command") -> bool:
+def _uses_process_substitution(cmd: "shellous.Command[Any]") -> bool:
     "Return True if command uses process substitution."
     return any(
         isinstance(arg, (shellous.Command, shellous.Pipeline)) for arg in cmd.args
     )
 
 
-def _is_multiple_capture(cmd):
+def _is_multiple_capture(cmd: "shellous.Command[Any]"):
     "Return true if both stdout and stderr are CAPTURE."
     output = Redirect.from_default(cmd.options.output, 1, cmd.options.pty)
     error = Redirect.from_default(cmd.options.error, 2, cmd.options.pty)
     return output == Redirect.CAPTURE and error == Redirect.CAPTURE
 
 
-def _cleanup(command):
+def _cleanup(command: "Union[shellous.Command[Any], shellous.Pipeline[Any]]"):
     "Close remaining file descriptors that need to be closed."
 
-    def _add_close(close, fdesc):
+    def _add_close(close: bool, fdesc: Any):
         if close:
             if isinstance(fdesc, (int, io.IOBase)):
                 open_fds.append(fdesc)
 
-    open_fds = []
+    open_fds: list[Any] = []
 
     _add_close(command.options.input_close, command.options.input)
     _add_close(command.options.output_close, command.options.output)
@@ -1137,7 +1226,7 @@ def _cleanup(command):
     close_fds(open_fds)
 
 
-def _signame(signal):
+def _signame(signal: Any) -> str:
     "Return string name of signal."
     if signal is None:
         return "SIGKILL"  # SIGKILL (even on win32)
@@ -1145,3 +1234,17 @@ def _signame(signal):
         return signal.name
     except AttributeError:
         return str(signal)
+
+
+def _set_position(output: Any, append: bool):
+    "Truncate/seek output stream object."
+    if isinstance(output, bytearray):
+        if not append:
+            output.clear()
+    elif isinstance(output, io.IOBase):
+        if output.seekable():
+            if append:
+                output.seek(0, io.SEEK_END)
+            else:
+                output.truncate(0)
+                output.seek(0, io.SEEK_SET)
